from typing import Optional

from htmltools import TagAttrArg, tags


def emoji(emoji_name: str, *, class_: Optional[str] = None, **kwargs: TagAttrArg):
    return tags.em(data_emoji=f":{emoji_name}:", class_=class_, **kwargs)



from typing import Any, Dict


def squash_whitespace(txt: str) -> str:
    """Removes extra spaces, e.g. in generated semantic class names"""
    return " ".join(txt.split())


# Credit: https://github.com/rstudio/py-shiny/blob/a55a11161ab394f2dce579bf636f92b01133cc5f/shiny/_utils.py#L48 # noqa: E501
def drop_none(x: Dict[str, Any]) -> Dict[str, object]:
    return {k: v for k, v in x.items() if v is not None}



from htmltools import HTMLDependency, TagList, tags


def page_semantic(*args, title=None, lang="en"):
    title = title or "Semantic App"
    head = TagList(
        tags.title(title),
        _semantic_dependency(),
        _shiny_semantic_bindings(),
    )
    body = TagList(*args)
    return tags.html(
        tags.head(head),
        tags.body(body),
        lang=lang,
        style="scroll-behavior: smooth;",
    )


def _semantic_dependency():
    return HTMLDependency(
        name="semantic-ui",
        version="2.9.0",
        source={"package": "shiny_semantic", "subdir": "www/semantic"},
        script=[{"src": "fomantic.min.js"}],
        stylesheet=[{"href": "fomantic.min.css"}],
    )


def _shiny_semantic_bindings():
    return HTMLDependency(
        name="shiny-semantic-bindings",
        version="0.0.1",
        source={"package": "shiny_semantic", "subdir": "www"},
        script=[{"src": "shiny-semantic-bindings.js", "type": "module"}],
    )



# Semantic `Modules` section, e.g. Accordion, Checkbox, etc
__all__ = (
    "checkbox_group",
    "checkbox",
    "dropdown",
    "input_select",
    "modal_show",
    "modal",
    "slider",
    "update_checkbox",
    "update_checkbox_group",
    "update_select",
    "update_slider",
)

from .checkbox import checkbox, checkbox_group, update_checkbox, update_checkbox_group
from .dropdown import dropdown, input_select, update_select
from .modal import modal, modal_show
from .slider import slider, update_slider



from typing import Optional

from htmltools import TagAttrArg, TagChildArg, tags

from shiny_semantic._utils import squash_whitespace


def statistic(
    value: TagChildArg,
    label: str,
    *,
    value_first=True,
    class_: Optional[str] = None,
    **kwargs: TagAttrArg,
):
    value_element = tags.div(value, class_="value")
    label_element = tags.div(label, class_="label")

    content = [label_element, value_element]

    if value_first:
        content.reverse()

    return tags.div(
        *content,
        class_=squash_whitespace(f"ui {class_ or ''} statistic"),
        **kwargs,
    )



from typing import Optional

from htmltools import TagAttrArg, tags

from shiny_semantic._utils import squash_whitespace


def icon(
    icon_name: str,
    *,
    class_: Optional[str] = None,
    **kwargs: TagAttrArg,
):
    return tags.i(
        class_=squash_whitespace(f"{class_ or ''} {icon_name} icon"),
        **kwargs,
    )



from typing import Optional, Union

from htmltools import TagList, tags
from shiny.module import resolve_id
from shiny.session import Session, require_active_session

from shiny_semantic._utils import drop_none, squash_whitespace


def checkbox(
    id: str,
    label: str,
    value: bool = False,
    *,
    type: Optional[str] = None,
    name: Optional[str] = None,
    class_: Optional[str] = None,
    **kwargs,
):

    class_ = f"{class_ or ''}"
    if value:
        class_ = f"{class_} checked"
    if type is not None:
        class_ = f"{class_} {type}"

    input_tag_type = "checkbox"
    if type == "radio":
        input_tag_type = "radio"

    # NOTE: if the value=False, let checked_ be None, i.e. not appear in HTML
    checked = "" if value else None

    id = resolve_id(id)
    name = name or id

    return tags.div(
        tags.input(id=id, type_=input_tag_type, name=name, checked=checked),
        tags.label(label, for_=id),
        id=id,
        class_=squash_whitespace(f"ui {class_} checkbox"),
        **kwargs,
    )


def checkbox_group(
    id: str,
    label: str,
    choices: Union[list[str], dict[str, str]],
    *,
    selected: Optional[Union[list[str], tuple[str], str]] = None,
    type: Optional[str] = None,
    position: Optional[str] = "grouped",
    class_: Optional[str] = None,
):
    if isinstance(choices, list):
        choices = {choice: choice for choice in choices}

    if selected is None:
        selected = []
    if isinstance(selected, str):
        selected = [selected]

    if type == "radio" and len(selected) > 1:
        raise Exception("Radio buttons may have a maximum of 1 active value")

    checkbox_tags = TagList()
    for v, l in choices.items():
        checkbox_tag = tags.div(
            checkbox(
                # NOTE: for checkbox_group to work correctly, all individual checkboxes
                # inside it must share a common name attribute.
                id=f"{id}__{v}",
                label=l,
                value=v in selected,
                type=type,
                name=id,
                data_shiny_value=v,
            ),
            class_="field",
        )
        checkbox_tags.append(checkbox_tag)

    id = resolve_id(id)
    class_ = squash_whitespace(f"{position} {class_ or ''} fields ss-checkbox-group")

    return tags.div(
        tags.div(
            tags.label(label),
            checkbox_tags,
            id=id,
            class_=class_,
        ),
        class_="ui form",
    )


def update_checkbox(
    id: str,
    *,
    label: Optional[str] = None,
    value: Optional[bool] = None,
    session: Optional[Session] = None,
):
    session = require_active_session(session)
    msg = {"label": label, "value": value}
    session.send_input_message(id, drop_none(msg))


def update_checkbox_group(
    id: str,
    *,
    labels: Optional[list[str]] = None,
    values: Optional[list[bool]] = None,
    group_label: Optional[str] = None,
    session: Optional[Session] = None,
):
    session = require_active_session(session)
    msg = {"labels": labels, "values": values, "group_label": group_label}
    session.send_input_message(id, drop_none(msg))



import json
from typing import Optional, Union

from htmltools import tags
from shiny.module import resolve_id
from shiny.session import Session, require_active_session

from shiny_semantic._utils import drop_none, squash_whitespace
from shiny_semantic.elements import icon


def dropdown(
    id: str,
    choices: list[str],
    *,
    value: Optional[Union[str, list[str]]] = None,
    placeholder: Optional[str] = None,
    settings: Optional[dict] = None,
    class_: Optional[str] = None,
):
    choice_tags = [
        tags.div(
            choice,
            class_="item",
            data_value=choice,
        )
        for choice in choices
    ]

    class_name = squash_whitespace(f"ui {class_ or ''} selection dropdown semantic-select-input")

    id = resolve_id(id)

    if isinstance(value, list):
        value = ",".join(value)

    # NOTE: This HTML structure represents the Fomantic-recomended way of creating this component
    # See: https://fomantic-ui.com/modules/dropdown.html#/usage
    return tags.div(
        tags.input(type_="hidden", name=id, value=value),
        icon("dropdown"),
        tags.div(placeholder, class_="default text"),
        tags.div(*choice_tags, class_="menu"),
        id=id,
        class_=class_name,
        data_settings=json.dumps(settings),
    )


def input_select(
    id: str,
    label: Optional[str],
    choices: list[str],
    *,
    selected: Optional[Union[str, list[str]]] = None,
    placeholder: Optional[str] = None,
    settings: Optional[dict] = None,
    class_: Optional[str] = None,
):
    id = resolve_id(id)
    return tags.div(
        tags.div(
            # NOTE: label tag is inspired by the way it is implemented in Shiny.
            # See: `from shiny.ui._utils import shiny_input_label`
            tags.label(label, id=f"{id}-label", for_=id),
            dropdown(
                id,
                choices,
                value=selected,
                placeholder=placeholder,
                settings=settings,
                class_=class_,
            ),
            class_="field",
        ),
        class_="ui form",
    )


def update_select(
    id: str,
    *,
    label: Optional[str] = None,
    choices: Optional[list[str]] = None,
    selected: Optional[Union[str, list[str]]] = None,
    session: Optional[Session] = None,
):
    session = require_active_session(session)

    msg_choices = None
    if choices is not None:
        msg_choices = {
            "values": [{"value": choice, "text": choice, "name": choice} for choice in choices]
        }

    msg = {
        "label": label,
        "choices": msg_choices,
        "value": selected,
    }

    session.send_input_message(id, drop_none(msg))



# Semantic `Elements` section, e.g. Button, Flag, Icon, etc
__all__ = (
    "button",
    "container",
    "divider",
    "emoji",
    "flag",
    "header",
    "icon",
    "segment",
    "subheader",
    "update_button",
    "semantic_input",
)

from .button import button, update_button
from .container import container
from .divider import divider
from .emoji import emoji
from .flag import flag
from .header import header, subheader
from .icon import icon
from .input import semantic_input
from .segment import segment



import re
from typing import Dict, Optional

from htmltools import Tag, TagAttrArg, TagChildArg, tags
from shiny.module import resolve_id
from shiny.session import Session, require_active_session

from shiny_semantic._utils import squash_whitespace
from shiny_semantic.elements import button


def modal(
    id: str,
    *,
    header: TagChildArg,
    content: TagChildArg,
    actions: Optional[TagChildArg] = None,
    class_: Optional[str] = None,
    **kwargs: TagAttrArg,
):
    modal_header = tags.div(header, class_="header")
    modal_content = tags.div(content, class_="content")
    modal_actions = (
        tags.div(actions, class_="actions")
        if actions
        else tags.div(
            button("cancel", "Cancel", class_="negative"),
            button("ok", "OK", class_="positive"),
            class_="actions",
        )
    )

    _assert_modal_actions(modal_actions)

    modal = tags.div(
        modal_header,
        modal_content,
        modal_actions,
        id=resolve_id(id),
        class_=squash_whitespace(f"ui {class_ or ''} modal"),
        **kwargs,
    )

    return modal


async def modal_show(
    modal_ui: Tag,
    modal_props: Optional[Dict] = None,
    shiny_input: str = "modal",
    session: Optional[Session] = None,
):
    session = require_active_session(session)

    await session.send_custom_message(
        type="showSemanticModal",
        message={
            "ui": str(modal_ui),
            "props": modal_props,
            "shiny_input": session.ns(shiny_input),
        },
    )


def _assert_modal_actions(actions_ui: Tag):
    match_approve = re.search("<button.*class=.*(approve).*>", str(actions_ui))
    match_positive = re.search("<button.*class=.*(positive).*>", str(actions_ui))
    match_deny = re.search("<button.*class=.*(deny).*>", str(actions_ui))
    match_negative = re.search("<button.*class=.*(negative).*>", str(actions_ui))

    if any([match_approve, match_positive, match_deny, match_negative]):
        return

    raise Exception(
        """Modal actions has to have at least one button with one
        of the following classes: positive, negative, approve, deny."""
    )



from typing import Optional

from htmltools import TagAttrArg, tags

from shiny_semantic._utils import squash_whitespace


def flag(country: str, *, class_: Optional[str] = None, **kwargs: TagAttrArg):
    return tags.i(
        class_=squash_whitespace(f"{class_ or ''} {country} flag"),
        **kwargs,
    )



from typing import Optional

from htmltools import TagAttrArg, TagChildArg, tags
from shiny.module import resolve_id
from shiny.session import Session, require_active_session

from shiny_semantic._utils import drop_none, squash_whitespace


def button(
    id: str,
    label: Optional[str] = None,
    *,
    icon: TagChildArg = None,
    class_: Optional[str] = None,
    **kwargs: TagAttrArg,
):
    class_name = f"ui {class_ or ''} button"

    return tags.button(
        icon,
        label,
        id=resolve_id(id),
        class_=squash_whitespace(class_name),
        **kwargs,
    )


def update_button(
    input_id: str,
    *,
    label: Optional[str] = None,
    icon_name: Optional[str] = None,
    session: Optional[Session] = None,
):
    session = require_active_session(session)
    msg = {
        "label": label,
        "icon": icon_name,
    }
    session.send_input_message(input_id, drop_none(msg))



from typing import Optional

from htmltools import TagAttrArg, TagChildArg, tags

from shiny_semantic._utils import squash_whitespace

# TODO: there are two header options implemented in Semantic UI:
# as a heading (h1, h2, etc) and as a div. The former scales in rems
# while the latter in ems. It is easier to implement the latter, since
# no overloads or conditionals will be required, but we should think
# whether it makes sense to leave only this option.


def header(
    *children: TagChildArg,
    class_: Optional[str] = None,
    **kwargs: TagAttrArg,
):
    return tags.div(
        *children,
        class_=squash_whitespace(f"ui {class_ or ''} header"),
        **kwargs,
    )


def subheader(
    *children: TagChildArg,
    class_: Optional[str] = None,
    **kwargs: TagAttrArg,
):
    return tags.div(
        *children,
        class_=squash_whitespace(f"{class_ or ''} sub header"),
        **kwargs,
    )



from typing import Optional, Union

from htmltools import TagAttrArg, TagChildArg, tags
from shiny.module import resolve_id

from shiny_semantic._utils import squash_whitespace


def semantic_input(
    id: str,
    # FIXME: adding label param to match Shiny API.
    # Should be revised -- is probably resolved, when
    # a shiny-like `shiny_semantic.elements.input_text()`
    # is created.
    label=None,
    value: Union[str, float] = "",
    *,
    placeholder: Optional[str] = "",
    icon: TagChildArg = None,
    type: str = "text",
    semantic_class: Optional[str] = None,
    semantic_label: Optional[str] = None,
    **kwargs: TagAttrArg,
):
    """Keword arguments (**kwargs) include all html attributes
    relevant to the input tag, including, for example, `min`, `max` and `step`
    in case of input type="number", as well as `class_` that is passed directly
    to the input tag, as opposed to the `semantic_class` that is passed to the
    enclosing div element.
    """
    # Enclosing div's class
    if semantic_class is None:
        semantic_class = ""

    # Modify the enclosing div's class if icon is passed
    if icon is not None:
        semantic_class += " icon"

    # Define the label
    label_tag = None
    if semantic_label is not None:
        label_tag = tags.div(semantic_label, class_="ui label")
        semantic_class += " labeled"

    # Finalize & clean the div's class
    semantic_class = squash_whitespace(f"ui {semantic_class} input")

    return tags.div(
        label_tag,
        tags.input(
            id=resolve_id(id),
            type=type,
            value=value,
            placeholder=placeholder,
            **kwargs,
        ),
        icon,
        class_=semantic_class,
    )



from typing import Optional

from htmltools import TagAttrArg, TagChildArg, tags

from shiny_semantic._utils import squash_whitespace


def container(
    *children: TagChildArg,
    class_: Optional[str] = None,
    **kwargs: TagAttrArg,
):
    return tags.div(
        *children,
        class_=squash_whitespace(f"ui {class_ or ''} container"),
        **kwargs,
    )



import json
from typing import Optional, Union

from htmltools import TagAttrArg, tags
from shiny.module import resolve_id
from shiny.session import Session, require_active_session

from shiny_semantic._utils import drop_none, squash_whitespace


def slider(
    id: str,
    # TODO: add `label: str` argument
    min_value: Union[float, int],
    max_value: Union[float, int],
    start_value: Union[float, int],
    *,
    end_value: Optional[Union[float, int]] = None,
    step: Optional[Union[float, int]] = None,
    show_labels: bool = True,
    show_ticks: bool = False,
    custom_labels: Optional[list[str]] = None,
    class_: Optional[str] = None,
    **kwargs: TagAttrArg,
):
    """
    By default, single-value slider is created.
    If `end_value` is passed, then the range-value slider is created.
    """

    class_ = class_ or ""

    if show_labels:
        class_ = f"{class_} labeled"

    if show_ticks:
        class_ = f"{class_} ticked"

    if end_value is not None:
        class_ = f"{class_} range"

    return tags.div(
        id=resolve_id(id),
        class_=squash_whitespace(f"ui {class_} slider ss-slider"),
        data_min_=min_value,
        data_max=max_value,
        data_start=start_value,
        data_end=end_value,
        data_step=step,
        data_labels=json.dumps(custom_labels),
        **kwargs,
    )


def update_slider(
    id: str,
    *,
    value: Union[int, float, list[float], list[int]],
    session: Optional[Session] = None,
):
    if isinstance(value, (int, float)):
        value = [value]

    session = require_active_session(session)
    msg = {"value": value}
    session.send_input_message(id, drop_none(msg))
